/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 *
 * This is the main source file of OwlPS ResultReader-UDP, a simple
 * program demonstrating the use of libowlps-resultreader-udp.
 * The received results are printed on the standard output.
 *
 * This program must be linked against libowlps and its dependencies
 * (see the Makefile).
 */


#include "owlps-resultreader.h"

#include <signal.h>
#include <unistd.h>

int main(void)
{
  struct sigaction action ; // Signal handler structure
  owl_result *result ;
  int sockfd ;

  /* Set up signal handlers */
  action.sa_flags = 0 ;
  sigemptyset(&action.sa_mask) ;
  action.sa_handler = owl_sigint_handler ;
  sigaction(SIGINT, &action, NULL) ;
  action.sa_handler = owl_sigterm_handler ;
  sigaction(SIGTERM, &action, NULL) ;

  /* Open the socket */
  sockfd = owl_create_udp_listening_socket(OWL_DEFAULT_RESULT_PORT) ;
  if (sockfd < 0)
    return 1 ;

  /* Read loop */
  owl_run = owl_true ;
  while (owl_run)
    {
      if (! (result = owl_receive_position(sockfd)))
        return 1 ;
      owl_print_result(result) ;
      owl_free_result(result) ;
      printf("--------------\n") ;
    }

  /* Cleaning */
  close(sockfd) ;

  return 0 ;
}
