/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#include "calibrationrequest.hh"
#include "referencepoint.hh"



/* *** Constructors *** */


CalibrationRequest::
CalibrationRequest(uint_fast8_t _type = OWL_REQUEST_AUTOCALIBRATION):
  reference_point(NULL), direction(Direction())
{
  type = _type ;
}


CalibrationRequest::
CalibrationRequest(const Request &source,
                   ReferencePoint *_reference_point = NULL,
                   const Direction &_direction = Direction(),
                   uint_fast8_t _type = OWL_REQUEST_AUTOCALIBRATION):
  Request(source), reference_point(_reference_point),
  direction(_direction)
{
  type = _type ;
}



/* *** Write accessors *** */


void CalibrationRequest::reference_point_delete_requests() const
{
  reference_point->delete_requests() ;
}


void CalibrationRequest::reference_point_backward_link() const
{
  reference_point->add_request(this) ;
}


void CalibrationRequest::clear()
{
  Request::clear() ;
  reference_point = NULL ;
  direction.clear() ;
}



/* *** Operators *** */


CalibrationRequest& CalibrationRequest::
operator=(const CalibrationRequest &source)
{
  if (this == &source)
    return *this ;

  this->Request::operator=(source) ;
  direction = source.direction ;
  reference_point = source.reference_point ;

  return *this ;
}


bool CalibrationRequest::
operator==(const CalibrationRequest &source) const
{
  if (this == &source)
    return true ;

  return
    this->Request::operator==(source) &&
    direction == source.direction &&
    reference_point == source.reference_point ;
}



size_t hash_value(const CalibrationRequest &source)
{
  size_t seed = 0 ;

  boost::hash_combine(seed, static_cast<Request>(source)) ;
  boost::hash_combine(seed, source.direction) ;
  if (source.reference_point != NULL)
    boost::hash_combine(seed, *source.reference_point) ;

  return seed ;
}
