/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_CALIBRATIONREQUEST_HH_
#define _OWLPS_POSITIONING_CALIBRATIONREQUEST_HH_

class ReferencePoint ;

#include "request.hh"
#include "direction.hh"

/// Represents a calibration Request sent by a mobile
class CalibrationRequest: public Request
{
protected:
  /// Reference point that the mobile calibrates
  ReferencePoint *reference_point ;
  /// Direction in which the mobile was during the measurement
  Direction direction ;

public:
  CalibrationRequest(uint_fast8_t _type) ;

  CalibrationRequest(const CalibrationRequest &source):
    Request(source), reference_point(source.reference_point),
    direction(source.direction) {}

  CalibrationRequest(const Request &source,
                     ReferencePoint *_reference_point,
                     const Direction &_direction,
                     uint_fast8_t _type) ;

  ~CalibrationRequest(void) {}

  /** @name Read accessors */
  //@{
  const Direction& get_direction(void) const ;
  ReferencePoint* get_reference_point(void) const ;
  //@}

  /** @name Write accessors */
  //@{
  void set_direction(const Direction &_direction) ;
  void set_reference_point(const ReferencePoint *_rp) ;
  /// Adds the CalibrationRequest to the #reference_point list of requests
  void reference_point_backward_link(void) const ;
  /// Deletes all the requests of #reference_point
  void reference_point_delete_requests(void) const ;
  void clear(void) ;
  //@}

  /** @name Operators */
  //@{
  CalibrationRequest& operator=(const CalibrationRequest &source) ;
  bool operator==(const CalibrationRequest &source) const ;
  bool operator!=(const CalibrationRequest &source) const ;
  //@}

  /// Hashes a CalibrationRequest
  friend std::size_t hash_value(const CalibrationRequest &source) ;
} ;



/* *** Read accessors *** */


inline ReferencePoint* CalibrationRequest::get_reference_point(void) const
{
  return reference_point ;
}


inline const Direction& CalibrationRequest::get_direction(void) const
{
  return direction ;
}



/* *** Write accessors *** */


inline void CalibrationRequest::set_reference_point(
  const ReferencePoint *_rp)
{
  reference_point = const_cast<ReferencePoint*>(_rp) ;
}


inline void CalibrationRequest::
set_direction(const Direction &_direction)
{
  direction = _direction ;
}



/* *** Operators *** */


inline bool CalibrationRequest::
operator!=(const CalibrationRequest &source) const
{
  return !(*this == source) ;
}



#endif // _OWLPS_POSITIONING_CALIBRATIONREQUEST_HH_
