/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#include "configuration.hh"

using namespace std ;
namespace po = boost::program_options ;



/* *** Attribute definitions *** */

po::variables_map Configuration::configuration ;



/* *** Read accessors *** */


po::variables_map& Configuration::getw_configuration()
{
  return configuration ;
}


bool Configuration::is_configured(const string &key)
{
  if (configuration.count(key))
    return true ;
  return false ;
}


const std::string&
Configuration::string_value(const string &key)
{
  return configuration[key].as<string>() ;
}


int Configuration::int_value(const string &key)
{
  return configuration[key].as<int>() ;
}


unsigned int Configuration::uint_value(const string &key)
{
  return configuration[key].as<unsigned int>() ;
}


float Configuration::float_value(const string &key)
{
  return configuration[key].as<float>() ;
}


bool Configuration::bool_value(const string &key)
{
  return configuration[key].as<bool>() ;
}


/**
 * @return \em true if the string \em value exists in the vector \em key.
 * @return \em false if \em value is not found in \em key.
 */
bool Configuration::
value_exists_in_string_vector(const string &key, const string &value)
{
  if (! is_configured(key))
    return false ;

  vector<string> key_values = configuration[key].as< vector<string> >() ;

  for (vector<string>::const_iterator i = key_values.begin() ;
       i != key_values.end() ; ++i)
    if (*i == value)
      return true ;

  return false ;
}


const vector<string>& Configuration::
string_vector_value(const string &key)
{
  return configuration[key].as< vector<string> >() ;
}
