/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_CONFIGURATION_HH_
#define _OWLPS_POSITIONING_CONFIGURATION_HH_

#include <string>
#include <vector>
#include <boost/program_options/variables_map.hpp>

/// Stocks the configuration of the program
class Configuration
{
private:
  /// Configuration structure
  static boost::program_options::variables_map configuration ;

public:
  /// Returns a reference to the configuration structure
  static boost::program_options::variables_map& getw_configuration(void) ;
  /// Checks if the key exists in the configuration
  static bool is_configured(const std::string &key) ;
  /// Returns the string value corresponding to \em key
  static const std::string& string_value(const std::string &key) ;
  /// Returns the int value corresponding to \em key
  static int int_value(const std::string &key) ;
  /// Returns the unsigned int value corresponding to \em key
  static unsigned int uint_value(const std::string &key) ;
  /// Returns the float value corresponding to \em key
  static float float_value(const std::string &key) ;
  /// Returns the bool value corresponding to \em key
  static bool bool_value(const std::string &key) ;
  /// Checks if a given string value exists in \em key
  static bool value_exists_in_string_vector(
    const std::string &key,
    const std::string &value) ;
  /// Returns the vector value corresponding to \em key
  static const std::vector<std::string>&
  string_vector_value(const std::string &key) ;
} ;

#endif // _OWLPS_POSITIONING_CONFIGURATION_HH_
