/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_CSVFILEREADER_HH_
#define _OWLPS_POSITIONING_CSVFILEREADER_HH_

class Point3D ;

#include "textfilereader.hh"
#include "timestamp.hh"

#include <boost/tokenizer.hpp>
#include <boost/lexical_cast.hpp>

/// Reads a CSV file, line by line, field by field
class CSVFileReader: public TextFileReader
{
protected:
  const char separator ;

  std::string current_line ;
  boost::tokenizer<boost::escaped_list_separator<char> > *current_token ;
  boost::tokenizer<boost::escaped_list_separator<char> >::const_iterator
  token_iterator ;
  unsigned int current_field_nb ;

  void print_error_cast(void) const ;

public:
  CSVFileReader(const std::string &filename,
                const char _separator = ';'):
    TextFileReader(filename), separator(_separator),
    current_token(NULL), current_field_nb(0) {}

  virtual ~CSVFileReader(void) ;

  /** @name Operations */
  //@{
  /// Loads the first next non-blank line
  bool next_line(void) ;
  /// Reads the next field of the current line
  template<class T> bool read_field(T &field) ;
  /// Reads the next field that should be a Timestamp
  bool read_timestamp(Timestamp &t) ;
  /// Reads the next 3 fields that should form a Point3D
  bool read_point3d(Point3D &p) ;
  //@}
} ;



/* *** Operations *** */


template<class T>
inline bool CSVFileReader::read_field(T &field)
{
  if (token_iterator == current_token->end())
    return false ;
  ++current_field_nb ;

  try
    {
      field = boost::lexical_cast<T>(*token_iterator) ;
    }
  catch (boost::bad_lexical_cast &e)
    {
      print_error_cast() ;
      return false ;
    }

  ++token_iterator ;
  return true ;
}



#endif // _OWLPS_POSITIONING_CSVFILEREADER_HH_
