/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#include "direction.hh"
#include "posexcept.hh"

#include <boost/functional/hash.hpp>



/* *** Constructors *** */


Direction::Direction(const char source):
  direction(source)
{
  assert_valid() ;
}



/* *** Operations *** */


inline void Direction::assert_valid() const
{
  if (! (is_valid() || direction == 0))
    throw bad_direction(direction) ;
}


inline bool Direction::is_valid() const
{
  return direction > 0 && direction <= 4 ;
}



/* *** Operators *** */


Direction& Direction::operator=(const Direction &source)
{
  direction = source.direction ;
  return *this ;
}


Direction& Direction::operator=(const char source)
{
  direction = source ;
  assert_valid() ;
  return *this ;
}


Direction::operator std::string() const
{
  switch (direction)
    {
    case north:
      return "north" ;
    case east:
      return "east" ;
    case south:
      return "south" ;
    case west:
      return "west" ;
    }
  return "Bad direction!" ;
}



size_t hash_value(const Direction &source)
{
  return boost::hash_value(source.direction) ;
}
