/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#include "fbcm.hh"
#include "stock.hh"
#include "configuration.hh"



Result FBCM::compute(const Request &_request)
{
  // If the autocalibration is activated, we have to regenerate the
  // Friis indexes each time we calculate a position
  if (Configuration::bool_value("positioning.generate-reference-points"))
    Stock::update_all_friis_indexes() ;
  return TrilaterationAlgorithm::compute(_request) ;
}


float FBCM::estimate_distance(const Measurement &measurement)
{
  double constant_term = make_constant_term(measurement) ;
  const float &average_dbm = measurement.get_average_dbm() ;
  const AccessPoint *ap = measurement.get_ap() ;
  return pow(10, (constant_term - average_dbm) /
             (10 * friis_index(ap))) ;
}


inline float FBCM::friis_index(const AccessPoint *const ap) const
{
  return ap->get_friis_index() ;
}
