/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_INPUTCSV_HH_
#define _OWLPS_POSITIONING_INPUTCSV_HH_

#include "inputmedium.hh"
#include "csvfilereader.hh"

#include <string>

/// Reads \link Request requests \endlink from a CSV file
/**
 * CSV format is:
 *   Mobile_MAC;Request_type;Number_of_packets;Timestamp;X;Y;Z;Direction;
 *     AP_MAC_1;Packet_ID_1;SS_1;…;AP_MAC_n;Packet_ID_n;SS_n
 */
class InputCSV: public InputMedium
{
protected:
  CSVFileReader file ;

  /** @name Operations */
  //@{
  /// Reads data & fills the current request
  bool fill_current_request(void) ;
  //@}

public:
  InputCSV(const std::string &filename):
    file(filename) {}

  ~InputCSV(void) {}

  /** @name Read accessors */
  //@{
  /// Checks if the end of the CSV file has been reached
  bool eof(void) const ;
  //@}

  /** @name Operators */
  //@{
  operator bool(void) const ;
  //@}
} ;



/* *** Read accessors *** */


inline bool InputCSV::eof() const
{
  return file.eof() ;
}



/* *** Operators *** */


inline InputCSV::operator bool() const
{
  return ! eof() ;
}



#endif // _OWLPS_POSITIONING_INPUTCSV_HH_
