/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_INPUTMEDIUM_HH_
#define _OWLPS_POSITIONING_INPUTMEDIUM_HH_

class ReferencePoint ;
class Direction ;

#include "request.hh"

#include <string>

/// Super class of all input media
/**
 * This class Provides an interface for input media, i.e. to read
 * Request sent by a Mobile.
 */
class InputMedium
{
protected:
  Request *current_request ;
  /// Number of the current line proceeded
  unsigned long current_line_nb ;

  /** @name Operations */
  //@{
  /// Reads data & fills the current request
  virtual bool fill_current_request(void) = 0 ;
  /// \brief Checks if the current request is a calibration request and
  /// if so fills it with additionnal data
  void fill_calibration_request_data(const std::string &mac_mobile,
                                     Point3D &position,
                                     const Direction &direction,
                                     uint8_t type) ;
  //@}

public:
  InputMedium(void) ;

  virtual ~InputMedium(void) ;

  /** @name Read accessors */
  //@{

  const Request& get_current_request(void) const ;

  unsigned int get_current_line_nb(void) const ;

  /// Checks if the last request has been reached
  /**
   * @return \em true if the last request has been reached.
   * @return \em false if there is something more to read.
   */
  virtual bool eof(void) const = 0 ;

  //@} // End Read accessors

  /** @name Operations */
  //@{

  /// Reads the next request
  const Request& get_next_request(void) ;

  /// Converts #current_request into a CalibrationRequest
  void current_request_to_calibration_request(
    const ReferencePoint *const reference_point,
    const Direction &direction,
    uint_fast8_t request_type) ;
  /// Clears (reallocates to Request if needed) #current_request
  void clear_current_request(void) ;

  //@} // End Operations
} ;



/* *** Read accessors *** */


inline const Request& InputMedium::get_current_request() const
{
  return *current_request ;
}


inline unsigned int InputMedium::get_current_line_nb() const
{
  return current_line_nb ;
}



#endif // _OWLPS_POSITIONING_INPUTMEDIUM_HH_
