/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_MINMAX_HH_
#define _OWLPS_POSITIONING_MINMAX_HH_

#include "trilaterationmethod.hh"

/// Multilaterates using the Lassabe's MinMax algorithm
class MinMax: public TrilaterationMethod
{
private:
  float min ;
  Point3D centre ;
  std::tr1::unordered_map<AccessPoint*, float> const *ap_distances ;

  void iterate(float x, float y, float z) ;

protected:
  const Point3D start ;
  const Point3D stop ;
  float step ;

  static const float MINMAX_DEFAULT_STEP = 0.5 ;
  static const float INFINITE = 1000000 ;

public:
  MinMax(const Point3D &_start,
         const Point3D &_stop,
         const float _step = MINMAX_DEFAULT_STEP):
    min(INFINITE),
    start(_start), stop(_stop), step(_step) {}

  ~MinMax(void) {}

  Point3D trilaterate(
    const std::tr1::unordered_map<AccessPoint*, float> &_ap_distances) ;
  Point3D trilaterate_2d(
    const std::tr1::unordered_map<AccessPoint*, float> &_ap_distances,
    float z) ;
} ;

#endif // _OWLPS_POSITIONING_MINMAX_HH_
