/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#include "outputudpsocket.hh"
#include "request.hh"
#include "resultlist.hh"
#include "result.hh"
#include "posexcept.hh"

#include <owlps.h>

#include <cstdio> // For perror()

using namespace std ;



/* *** Constructors *** */


OutputUDPSocket::OutputUDPSocket(const string &_remote_host,
                                 const uint_fast16_t _remote_port):
  OutputNetworkSocket(_remote_host, _remote_port)
{
  if (! init_socket())
    throw error_opening_output_file("UDP socket") ;
}



/* *** Operations *** */


/**
 * @return \em true if the socket were successfully opened.
 * @return \em false in case of error.
 */
bool OutputUDPSocket::init_socket()
{
  sockfd = owl_create_udp_trx_socket(remote_host.c_str(), remote_port,
                                     &server_info, &client_info) ;
  return sockfd >= 0 ;
}


/**
 * Sends the text buffer 'data'.
 */
bool OutputUDPSocket::send_data(const string &data) const
{
  unsigned int data_len = data.size() + 1 ; // +1 for the '\0'
  ssize_t nsent = sendto(sockfd, data.c_str(), data_len, 0,
                         (struct sockaddr *) &server_info,
                         sizeof(server_info)) ;
  if (nsent != static_cast<ssize_t>(data_len))
    {
      perror("Error sending result data through the UDP socket") ;
      return false ;
    }
  return true ;
}
