/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#include "posexcept.hh"

#include <sstream>

using namespace std ;



/* *** Miscellaneous exceptions *** */


cannot_merge::
cannot_merge(const string &_explanation) throw():
  posexcept(_explanation) {}


element_not_found::
element_not_found(const string &_explanation) throw():
  posexcept("Element not found in collection: " + _explanation) {}


autocalibration_error::
autocalibration_error(const string &_explanation) throw():
  posexcept("Cannot generate reference points: " + _explanation) {}



/* *** Configuration *** */


missing_configuration::
missing_configuration(const string &_explanation) throw():
  posexcept(_explanation) {}


bad_configuration::
bad_configuration(const string &_explanation) throw():
  posexcept(_explanation) {}



/* *** Input *** */


malformed_input_data::
malformed_input_data(const string &_explanation) throw():
  posexcept("Read error: " + _explanation) {}


bad_direction::bad_direction(const char direction) throw()
{
  ostringstream message ;
  message << "`" << static_cast<int>(direction)
          << "` is not a valid direction value!" ;
  explanation = message.str() ;
}


bad_channel::bad_channel(const unsigned long channel) throw()
{
  ostringstream message ;
  message << "`" << channel
          << "` is not a valid channel value!" ;
  explanation = message.str() ;
}


null_input_medium::null_input_medium() throw():
  posexcept("The input medium is not initialised!") {}



/* *** Files *** */


error_opening_input_file::
error_opening_input_file(const string &file_name) throw():
  posexcept("Error opening input file \"" + file_name + "\"!") {}


error_opening_output_file::
error_opening_output_file(const string &file_name) throw():
  posexcept("Error opening output file \"" + file_name + "\"!") {}
