/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_POSEXCEPT_HH_
#define _OWLPS_POSITIONING_POSEXCEPT_HH_

#include <exception>
#include <string>



/// Super-class of all exceptions defined in OwlPS Positioner
class posexcept: public std::exception
{
protected:
  std::string explanation ;

public:
  posexcept(const std::string &_explanation = "") throw():
    explanation(_explanation) {}

  ~posexcept(void) throw() {}

  const char* what(void) const throw()
  {
    return explanation.c_str() ;
  }
} ;



/* *** Miscellaneous exceptions *** */


class cannot_merge: public posexcept
{
public:
  cannot_merge(const std::string &_explanation) throw() ;
} ;


class element_not_found: public posexcept
{
public:
  element_not_found(const std::string &_explanation) throw() ;
} ;


class autocalibration_error: public posexcept
{
public:
  autocalibration_error(const std::string &_explanation) throw() ;
} ;



/* *** Configuration *** */


class missing_configuration: public posexcept
{
public:
  missing_configuration(const std::string &_explanation) throw() ;
} ;


class bad_configuration: public posexcept
{
public:
  bad_configuration(const std::string &_explanation) throw() ;
} ;



/* *** Input *** */


class malformed_input_data: public posexcept
{
public:
  malformed_input_data(const std::string &_explanation) throw() ;
} ;


class bad_direction: public posexcept
{
public:
  bad_direction(const char direction) throw() ;
} ;


class bad_channel: public posexcept
{
public:
  bad_channel(const unsigned long channel) throw() ;
} ;


class null_input_medium: public posexcept
{
public:
  null_input_medium(void) throw() ;
} ;



/* *** Files *** */


class error_opening_input_file: public posexcept
{
public:
  error_opening_input_file(const std::string &file_name) throw() ;
} ;


class error_opening_output_file: public posexcept
{
public:
  error_opening_output_file(const std::string &file_name) throw() ;
} ;



#endif // _OWLPS_POSITIONING_POSEXCEPT_HH_
