/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#include "resultlist.hh"
#include "request.hh"
#include "mobile.hh"

#include <sstream>

using namespace std ;



/* *** Constructors *** */


ResultList::~ResultList()
{
  results.clear() ;
}



/* *** Operators *** */


ResultList& ResultList::operator=(const ResultList &source)
{
  if (this == &source)
    return *this ;

  request = source.request ;
  results = source.results ;

  return *this ;
}


bool ResultList::operator==(const ResultList &source) const
{
  return
    request == source.request &&
    results == source.results ;
}



/* *** Conversion accessors *** */


/**
 * This function creates a CSV string from the Result's data.
 *
 * The format used is the following:
 *   Mobile_MAC;Request_type;Timestamp;[Algorithm_1];…;[Algorithm_n]
 *
 * The CSV format for the [Algorithm_i] strings is documented in
 * Result::to_csv().
 *
 * @return The results as a CSV string, \em without a trailing '\\n'.
 */
const string ResultList::to_csv() const
{
  ostringstream csv_line ;

  if (request != NULL)
    {
      if (request->get_mobile() != NULL)
        csv_line << request->get_mobile()->get_mac_addr() ;
      csv_line
          << ';' << static_cast<uint_fast16_t>(request->get_type())
          << ';' << request->get_time_sent() ;
    }
  else
    csv_line << ";;;" ;

  for (vector<Result>::const_iterator r = results.begin() ;
       r != results.end() ; ++r)
    csv_line << ';' << r->to_csv() ;

  return csv_line.str() ;
}



ostream& operator<<(ostream &os, const ResultList &r)
{
  if (r.request == NULL)
    os << "For an unknown request:\n" ;
  else
    os << "For the following request: " << *(r.request) << '\n' ;

  vector<Result>::const_iterator res ;
  for (res = r.results.begin() ; res != r.results.end() ; ++res)
    os << *res << '\n' ;

  return os ;
}
