/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_RESULTLIST_HH_
#define _OWLPS_POSITIONING_RESULTLIST_HH_

class Request ;
#include "result.hh"

#include <vector>
#include <ostream>

/// \brief List of \link Result results \endlink of a single Request,
/// computed by one or more positioning algorithm
class ResultList
{
protected:
  Request *request ;
  std::vector<Result> results ;

public:
  ResultList(const Request *_request = NULL,
             const std::vector<Result> &_results = std::vector<Result>()):
    request(const_cast<Request*>(_request)), results(_results) {}
  ResultList(const ResultList &source):
    request(source.request), results(source.results) {}
  ~ResultList(void) ;

  /** @name Read accessors */
  //@{
  /// Checks if there is at least one Result in the list
  bool empty(void) const ;
  const Request* get_request(void) const ;
  const std::vector<Result>& get_results(void) const ;
  //@}

  /** @name Write accessors */
  //@{
  void add(const Result &res) ;
  //@}

  /** @name Operators */
  //@{
  ResultList& operator=(const ResultList &source) ;
  bool operator==(const ResultList &source) const ;
  bool operator!=(const ResultList &source) const ;
  //@}

  /** @name Conversion accessors */
  //@{
  /// Converts to a CSV string
  const std::string to_csv(void) const ;
  //@}

  /// Displays a ResultList
  friend std::ostream& operator<<(std::ostream &os,
                                  const ResultList &r) ;
} ;



/* *** Read accessors *** */


inline bool ResultList::empty() const
{
  return results.empty() ;
}


inline const Request* ResultList::get_request() const
{
  return request ;
}


inline const std::vector<Result>& ResultList::get_results(void) const
{
  return results ;
}




/* *** Write accessors *** */


inline void ResultList::add(const Result &res)
{
  results.push_back(res) ;
}



#endif // _OWLPS_POSITIONING_RESULTLIST_HH_
