/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_TEXTFILEREADER_HH_
#define _OWLPS_POSITIONING_TEXTFILEREADER_HH_

#include <string>
#include <fstream>

/// Reads text from a file, line by line
/**
 * If the file name is a dash ('-'), the standard input is used.
 */
class TextFileReader
{
protected:
  /// Name of the input file
  std::string file_name ;
  /// Stream associated with the file
  std::ifstream file ;
/// Original rdbuf of the stream
  std::streambuf *file_buf ;
  /// Number of the last line read
  unsigned int current_line_nb ;

  /// Checks if the file is readable and closes it if not
  bool eof_close(void) ;

public:
  TextFileReader(const std::string &_file_name) ;
  virtual ~TextFileReader(void) ;

  /** @name Operations */
  //@{
  /// Reads the next non-blank line
  bool read_nonblank_line(std::string &text) ;
  /// Reads the next line
  bool read_line(std::string &text) ;
  bool eof(void) const ;
  //@}
} ;



/* *** Operations *** */


/**
 * @return \em true if the end of file is reached.
 * @return \em false if there is something more to read.
 */
inline bool TextFileReader::eof() const
{
  return ! file && file.eof() ;
}



#endif // _OWLPS_POSITIONING_TEXTFILEREADER_HH_
