/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#include "textfilewriter.hh"
#include "configuration.hh"
#include "posexcept.hh"

#include <iostream>

using namespace std ;



/* *** Constructors *** */


/**
 * @param _file_name The name of the file to open ('-' for the standard
 * output).
 * @throw error_opening_input_file if the file cannot be opened.
 */
TextFileWriter::TextFileWriter(const string &_file_name):
  file_name(_file_name)
{
  if (file_name == "-")
    {
      file_buf = file.rdbuf() ;
      file.std::ios::rdbuf(std::cout.rdbuf()) ;
    }
  else
    {
      file.open(file_name.c_str()) ;
      if (! file)
        throw error_opening_output_file(file_name) ;
    }
}


TextFileWriter::~TextFileWriter()
{
  if (file_name == "-")
    file.std::ios::rdbuf(file_buf) ;
  else
    file.close() ;
}



/* *** Operations *** */


/**
 * @return \em true if the string has been written successfuly, or
 * \em false if not.
 */
bool TextFileWriter::write_text(const string &text)
{
  if (! file)
    return false ;

  file << text ;

  if (Configuration::bool_value("flush-output-files"))
    file.flush() ;

  return true ;
}
