/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#include "topologyreadercsv.hh"
#include "posexcept.hh"
#include "stock.hh"
#include "area.hh"
#include "waypoint.hh"

using namespace std ;



/* *** Constructors *** */


TopologyReaderCSV::TopologyReaderCSV(
  const string &areas_file_name, const string &waypoints_file_name):
  areas_file(areas_file_name), waypoints_file(waypoints_file_name)
{
  read_topology() ;
}



/* *** Operations *** */


void TopologyReaderCSV::read_topology()
{
  while (areas_file.next_line())
    process_area_line() ;

  while (waypoints_file.next_line())
    process_waypoint_line() ;
}


void TopologyReaderCSV::process_area_line()
{
  string building_name ;
  if (! areas_file.read_field(building_name))
    throw malformed_input_data("Cannot read building name!") ;
  Building &building = const_cast<Building&>(
                         Stock::find_create_building(building_name)) ;

  string name ;
  if (! areas_file.read_field(name))
    throw malformed_input_data("Cannot read area name!") ;

  Point3D p_min ;
  if (! areas_file.read_point3d(p_min))
    throw malformed_input_data("Cannot read area min coordinates!") ;

  Point3D p_max ;
  if (! areas_file.read_point3d(p_max))
    throw malformed_input_data("Cannot read area max coordinates!") ;

  Area *area = new Area(&building, name, p_min, p_max) ;
  building.add_area(area) ;
}


void TopologyReaderCSV::process_waypoint_line()
{
  Point3D coordinates ;
  if (! waypoints_file.read_point3d(coordinates))
    throw malformed_input_data("Cannot read waypoint coordinates!") ;

  Waypoint &waypoint_ref =
    const_cast<Waypoint&>(Stock::find_create_waypoint(coordinates)) ;

  vector<Building*> buildings ;
  string building_name ;
  while (waypoints_file.read_field(building_name))
    buildings.push_back(const_cast<Building*>(
                          &Stock::find_create_building(building_name))) ;
  if (buildings.empty())
    throw malformed_input_data("Cannot read building name!") ;

  for (vector<Building*>::iterator i = buildings.begin() ;
       i != buildings.end() ; ++i)
    {
      waypoint_ref.add_building(*i) ;
      (*i)->add_waypoint(&waypoint_ref) ;
    }
}
