/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_TOPOLOGYREADERCSV_HH_
#define _OWLPS_POSITIONING_TOPOLOGYREADERCSV_HH_

class Point3D ;

#include "csvfilereader.hh"

#include <string>

/// \brief Reads and registers to the Stock Area and Waypoint lists
/// from CSV files
/**
 * CSV format for areas is:
 *   Building name;Room name;X1;Y1;Z1;X2;Y2;Z2
 *
 * CSV format for waypoints is:
 *   X;Y;Z;Building name #1[;Building name #2[…[;Building name \#n]]]
 */
class TopologyReaderCSV
{
protected:
  CSVFileReader areas_file ;
  CSVFileReader waypoints_file ;

  void read_topology(void) ;
  void process_area_line(void) ;
  void process_waypoint_line(void) ;

public:
  TopologyReaderCSV(const std::string &areas_file_name,
                    const std::string &waypoints_file_name) ;

  ~TopologyReaderCSV(void) {}
} ;

#endif // _OWLPS_POSITIONING_TOPOLOGYREADERCSV_HH_
