/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_TRILATERATIONMETHOD_HH_
#define _OWLPS_POSITIONING_TRILATERATIONMETHOD_HH_

class AccessPoint ;

#include "point3d.hh"

#include <boost/tr1/unordered_map.hpp>

/// Super-class of all trilateration methods
/**
 * The source data is a list of access points associated with the
 * distances to the mobile. These distances are estimated using a
 * TrilaterationAlgorithm.
 */
class TrilaterationMethod
{
public:
  TrilaterationMethod(void) {}
  virtual ~TrilaterationMethod(void) {}

  /// Selects a point in 3D space
  virtual Point3D trilaterate(
    const std::tr1::unordered_map<AccessPoint*, float> &ap_distances) = 0 ;

  /// Selects a point in 2D space, given its vertical coordinate (z)
  virtual Point3D trilaterate_2d(
    const std::tr1::unordered_map<AccessPoint*, float> &ap_distances,
    float z) = 0 ;
} ;

#endif // _OWLPS_POSITIONING_TRILATERATIONMETHOD_HH_
