/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comte
 * (Université de Franche-Comté), France.
 *
 * Copyright © Université de Franche-Comté 2007-2012.
 *
 * Corresponding author: Matteo Cypriani <mcy@lm7.fr>
 *
 ***********************************************************************
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL:
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided
 * only with a limited warranty and the software's authors, the holder
 * of the economic rights, and the successive licensors have only
 * limited liability.
 *
 * In this respect, the user's attention is drawn to the risks
 * associated with loading, using, modifying and/or developing or
 * reproducing the software by the user in light of its specific status
 * of free software, that may mean that it is complicated to manipulate,
 * and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users
 * are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of
 * their systems and/or data to be ensured and, more generally, to use
 * and operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 ***********************************************************************
 */


#ifndef _OWLPS_POSITIONING_USERINTERFACE_HH_
#define _OWLPS_POSITIONING_USERINTERFACE_HH_

#include <string>
#include <boost/program_options/options_description.hpp>

/// Handles configuration inputs from user
class UserInterface
{
protected:
  int cli_argument_count ;
  char **cli_argument_values ;
  boost::program_options::options_description *cli_options ;
  boost::program_options::options_description *file_options ;

  /** @name Operations */
  //@{
  /// Fills in all the options
  void fill_options(void) ;

  /// Fills in the options accepted on the command line
  void fill_cli_options(void) ;

  /// \brief Fills in the options accepted on the command line and in
  /// the configuration file
  void fill_file_options(void) ;
  void fill_data_input_options(void) ;
  void fill_input_options(void) ;
  void fill_log_options(void) ;
  void fill_positioning_options(void) ;
  void fill_output_options(void) ;
  void fill_misc_options(void) ;

  /// Parses all the configuration inputs and updates Configuration
  void parse_options(void) ;
  void parse_command_line(void) const ;
  void parse_file(void) const ;

  /// Prints usage and/or version information if requested by the user
  void print_information(void) const ;
  //@}

public:
  UserInterface(const int argc, char **argv) ;

  ~UserInterface(void) ;
} ;

#endif // _OWLPS_POSITIONING_USERINTERFACE_HH_
