#include <cxxtest/TestSuite.h>

#include "inputcsv.hh"
#include "stock.hh"

#include <string>
#include <fstream>
#include <cstdio>

class InputCSV_test: public CxxTest::TestSuite
{
private:
  std::string csv_file_name ; // Test CSV file name
  std::vector<AccessPoint> aps ; // List of test AccessPoint


public:
  InputCSV_test(void)
  {
    // If we cannot open the file, we want to stop the test
    CxxTest::setAbortTestOnFail(true) ;

    // Clear the stock
    Stock::clear() ;

    csv_file_name = "/tmp/InputCSV_test_csv_file.csv" ;
    TestUtil::create_test_csv_file(csv_file_name, true) ;

    // Back to the normal behaviour (i.e. do not abort on fail)
    CxxTest::setAbortTestOnFail(false) ;
  }


  ~InputCSV_test(void)
  {
    // Delete the test CSV file
    TestUtil::remove_file(csv_file_name) ;

    // Clear the stock
    Stock::clear() ;
  }


  static InputCSV_test* createSuite(void)
  {
    return new InputCSV_test() ;
  }


  static void destroySuite(InputCSV_test *suite)
  {
    delete suite ;
  }


  void test_inputcsv(void)
  {
    // Opening file
    InputCSV inputcsv1(csv_file_name) ;
    TS_ASSERT(inputcsv1) ;
    TS_ASSERT(! inputcsv1.eof()) ;

    Request request1 ;

    // First Request //

    request1 = inputcsv1.get_next_request() ;
    TS_ASSERT(request1) ;
    assert(request1.get_mobile()) ;
    TS_ASSERT_EQUALS(request1.get_mobile()->get_mac_addr(),
                     TestUtil::mobiles[0].get_mac_addr()) ;
    TS_ASSERT_EQUALS(request1.get_time_sent(),
                     TestUtil::requests.at(0)->get_time_sent()) ;

    for (std::vector<AccessPoint>::const_iterator i = TestUtil::aps.begin() ;
         i != TestUtil::aps.end() ; ++i)
      {
        std::tr1::unordered_map<std::string, Measurement>::const_iterator
        measurement_it1 =
          request1.get_measurements().find(i->get_mac_addr()) ;
        TS_ASSERT(request1.get_measurements().end() != measurement_it1) ;
        TS_ASSERT_EQUALS(TestUtil::requests.at(0)->get_measurements().
                         at(i->get_mac_addr()).get_ss_list(),
                         measurement_it1->second.get_ss_list()) ;
      }

    TS_ASSERT(inputcsv1) ;
    TS_ASSERT(! inputcsv1.eof()) ;

    // Second Request //

    request1 = inputcsv1.get_next_request() ;
    TS_ASSERT(request1) ;
    assert(request1.get_mobile()) ;
    TS_ASSERT_EQUALS(request1.get_mobile()->get_mac_addr(),
                     TestUtil::mobiles[1].get_mac_addr()) ;
    TS_ASSERT_EQUALS(request1.get_time_sent(),
                     TestUtil::requests.at(1)->get_time_sent()) ;

    for (std::vector<AccessPoint>::const_iterator i = TestUtil::aps.begin() ;
         i != TestUtil::aps.end() ; ++i)
      {
        std::tr1::unordered_map<std::string, Measurement>::const_iterator
        measurement_it1 =
          request1.get_measurements().find(i->get_mac_addr()) ;
        TS_ASSERT(request1.get_measurements().end() != measurement_it1) ;
        TS_ASSERT_EQUALS(TestUtil::requests.at(1)->get_measurements().
                         at(i->get_mac_addr()).get_ss_list(),
                         measurement_it1->second.get_ss_list()) ;
      }

    // Third Request //

    TS_ASSERT(inputcsv1) ;
    TS_ASSERT(request1) ;
    assert(request1.get_mobile()) ;
    TS_ASSERT(! inputcsv1.eof()) ;

    request1 = inputcsv1.get_next_request() ;
    TS_ASSERT_EQUALS(request1.get_mobile()->get_mac_addr(),
                     TestUtil::mobiles[0].get_mac_addr()) ;
    TS_ASSERT_EQUALS(request1.get_time_sent(),
                     TestUtil::requests.at(2)->get_time_sent()) ;

    for (std::vector<AccessPoint>::const_iterator i = TestUtil::aps.begin() ;
         i != TestUtil::aps.end() ; ++i)
      {
        std::tr1::unordered_map<std::string, Measurement>::const_iterator
        measurement_it1 =
          request1.get_measurements().find(i->get_mac_addr()) ;
        TS_ASSERT(request1.get_measurements().end() != measurement_it1) ;
        TS_ASSERT_EQUALS(TestUtil::requests.at(2)->get_measurements().
                         at(i->get_mac_addr()).get_ss_list(),
                         measurement_it1->second.get_ss_list()) ;
      }

    // End of file

    TS_ASSERT(inputcsv1) ;
    TS_ASSERT(! inputcsv1.eof()) ;

    request1 = inputcsv1.get_next_request() ;
    TS_ASSERT(! request1) ;
    TS_ASSERT(! inputcsv1) ;
    TS_ASSERT(inputcsv1.eof()) ;
  }

} ;
